/*****************************************************************************
 File: clock.c

 Description:
 This file contains the clock functions.

 Authors and Copyright:
 (c) 2012, Thomas Fischl <tfischl@gmx.de>

 Device: PIC18F14K50
 Compiler: HI-TECH C PRO for the PIC18 MCU Family (Lite)  V9.65

 License:
 This file is open source. You can use it or parts of it in own
 open source projects. For closed or commercial projects you have to
 contact the authors listed above to get the permission for using
 this source code.

 ****************************************************************************/
//#include <htc.h>
//#include <stdio.h>
//#include <inttypes.h>
//#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/wdt.h>						// 2013.08.20
#include "clock.h"
// ----------------------------------------------------------------------------
#define	KEY1_DOWN			!(PING & 0x01)	//(PING.0 == 0) Selection
#define	KEY2_DOWN			!(PING & 0x02)	//(PING.1 == 0) GO / ENTER
#define	KEY3_DOWN			!(PING & 0x04)	//(PING.2 == 0) ESC / NORMAL
#define	SET_TIMER0_64		TCCR0 = (1 << CS02)
#define LED_PORT			0				// use PORTF.0
#define	LED_HIGH			PORTF |= _BV(LED_PORT)
#define	LED_LOW				PORTF &= ~_BV(LED_PORT)
// ----------------------------------------------------------------------------
int	ReadKey = 0, KeyDelay = 10, menu = 0;
//unsigned short clock_lastclock;
unsigned short clock_msticker;
unsigned short timeP2CAN = 0;
unsigned char  COUNT_TCNT0 = 0;				// to make 1 ms tick
unsigned char  PortLED = 0;
// ----------------------------------------------------------------------------
/**
 * Initialize & start clock/timer module
 */
void clock_start(void) {

	clock_msticker = 0;
	TCNT0 = COUNT_TCNT0;
	InitClockISR();
}
// ----------------------------------------------------------------------------
/**
 * Returns the milliseconds since last reset
 *
 * @return milliseconds since last reset
 */
unsigned short clock_getMS() {
    return clock_msticker;
}
// ============================================================================
/**
 * Handle clock task. Count milliseconds - void clock_process()
 */
ISR(TIMER0_OVF_vect) { 				// Global clock : 1 ms - Prescale = 64

	TCNT0 = COUNT_TCNT0;
	clock_msticker++;
	timeP2CAN++;					// in order to use in 'obd2can' for P2CAN
	if (clock_msticker > 60000) clock_msticker = 0;
	if(PortLED){
		PortLED = 0;	LED_HIGH;
	}else{
		PortLED = 1;	LED_LOW;
	}
	if (ReadKey == 0) {
        if (KeyDelay <= 0) {              // KeyDelay == 0
            KeyDelay = 0;
            if (KEY1_DOWN)	ReadKey = 1;
            if (KEY2_DOWN)	ReadKey = 2;
			if (KEY3_DOWN)	ReadKey = 3;
			if (ReadKey) 	KeyDelay = 300; 
        }else{
			KeyDelay--;
		}
	} 	
}
//void clock_process() {
//    while ((unsigned short) (TMR0 - clock_lastclock) > CLOCK_TIMERTICKS_1MS) {
//       clock_lastclock += CLOCK_TIMERTICKS_1MS;
//       clock_msticker++;
//       if (clock_msticker > 60000) clock_msticker = 0;
//    }
//}
// ----------------------------------------------------------------------------
void InitClockISR(void){

	// Enable Timer 0
	TIMSK = ((1 << TOIE0) | (1 << TOIE1) | (1 << TOIE2)); //  (1 << TOIE1)	
	// First set the GlobalClock in timer 0
	// T = (Prescale/clock)*(countmax) = (64/16,000,000)*(256-0) = 1ms
	TCNT0 = COUNT_TCNT0;
	SET_TIMER0_64;			// Use prescaler 64 for Global Time
	sei();					// MUST ! without this no ISR is activated !

}
//-----------------------------------------------------------------------------