/*****************************************************************************
 File: mcp2515.c
 Description:
 This file contains the MCP2515 interface functions.

 Authors and Copyright:
 (c) 2012, Thomas Fischl <tfischl@gmx.de>

 Device: PIC18F14K50
 Compiler: HI-TECH C PRO for the PIC18 MCU Family (Lite)  V9.65
 
 Ported to use in AVR : 2013.10.14 YJP
 Compiler: AVR GCC
 
 License:
 This file is open source. You can use it or parts of it in own
 open source projects. For closed or commercial projects you have to
 contact the authors listed above to get the permission for using
 this source code.
 ****************************************************************************/
// 2013.10.14 : Initial porting for use with AVR 
// ---------------------------------------------------------------------------
#include <stdio.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#include "mcp2515.h"
// ============================================================================
// 2013.10.14 : can_filter_XXX are added just for validation test only
// ----------------------------------------------------------------------------
prog_uint8_t can_filter_ECU [] ={ 
    // Group 0 
    MCP2515_FILTER (0x7e8), // ??Filter 0 - if '0' accept ALL message
    MCP2515_FILTER (0x7e8), // ??Filter 1 

    // Group 1 
    MCP2515_FILTER (0x7e8), // ??Filter 2 
    MCP2515_FILTER (0x7e8), // ??Filter 3 
    MCP2515_FILTER (0x7e8), // ??filter 4 
    MCP2515_FILTER (0x7e8), // ??Filter 5 

    MCP2515_FILTER (0x7FF), // Mask 0 (for group 0) 
    MCP2515_FILTER (0x7FF) // Mask 1 (for group 1) 
};
// ----------------------------------------------------------------------------
prog_uint8_t can_filter_TCU [] = { 
    // Group 0 
    MCP2515_FILTER (PID_REPLY_TCU), // ??Filter 0 - if '0' accept ALL message
    MCP2515_FILTER (PID_REPLY_TCU), // ??Filter 1 

    // Group 1 
    MCP2515_FILTER (PID_REPLY_TCU), // ??Filter 2 
    MCP2515_FILTER (PID_REPLY_TCU), // ??Filter 3 
    MCP2515_FILTER (PID_REPLY_TCU), // ??filter 4 
    MCP2515_FILTER (PID_REPLY_TCU), // ??Filter 5 

    MCP2515_FILTER (0x7FF), // Mask 0 (for group 0) 
    MCP2515_FILTER (0x7FF) // Mask 1 (for group 1) 
};
// ----------------------------------------------------------------------------
prog_uint8_t can_filter_HCU [] = { 
    // Group 0 
    MCP2515_FILTER (PID_REPLY_HCU), // ??Filter 0 - if '0' accept ALL message
    MCP2515_FILTER (PID_REPLY_HCU), // ??Filter 1 

    // Group 1 
    MCP2515_FILTER (PID_REPLY_HCU), // ??Filter 2 
    MCP2515_FILTER (PID_REPLY_HCU), // ??Filter 3 
    MCP2515_FILTER (PID_REPLY_HCU), // ??filter 4 
    MCP2515_FILTER (PID_REPLY_HCU), // ??Filter 5 

    MCP2515_FILTER (0x7FF), // Mask 0 (for group 0) 
    MCP2515_FILTER (0x7FF) // Mask 1 (for group 1) 
};
// ----------------------------------------------------------------------------
prog_uint8_t can_filter_MCU [] = { 
    // Group 0 
    MCP2515_FILTER (PID_REPLY_MCU), // ??Filter 0 - if '0' accept ALL message
    MCP2515_FILTER (PID_REPLY_MCU), // ??Filter 1 

    // Group 1 
    MCP2515_FILTER (PID_REPLY_MCU), // ??Filter 2 
    MCP2515_FILTER (PID_REPLY_MCU), // ??Filter 3 
    MCP2515_FILTER (PID_REPLY_MCU), // ??filter 4 
    MCP2515_FILTER (PID_REPLY_MCU), // ??Filter 5 

    MCP2515_FILTER (0x7FF), // Mask 0 (for group 0) 
    MCP2515_FILTER (0x7FF) // Mask 1 (for group 1) 
};
// ----------------------------------------------------------------------------
prog_uint8_t can_filter_BMS [] = { 
    // Group 0 
    MCP2515_FILTER (PID_REPLY_BMS), // ??Filter 0 - if '0' accept ALL message
    MCP2515_FILTER (PID_REPLY_BMS), // ??Filter 1 

    // Group 1 
    MCP2515_FILTER (PID_REPLY_BMS), // ??Filter 2 
    MCP2515_FILTER (PID_REPLY_BMS), // ??Filter 3 
    MCP2515_FILTER (PID_REPLY_BMS), // ??filter 4 
    MCP2515_FILTER (PID_REPLY_BMS), // ??Filter 5 

    MCP2515_FILTER (0x7FF), // Mask 0 (for group 0) 
    MCP2515_FILTER (0x7FF) // Mask 1 (for group 1) 
};
// ===========================================================================
/**
 * \brief Transmit one byte over SPI bus
 *
 * \param c Character to send
 * \return Received byte
 */
unsigned char spi_transmit(unsigned char c) {	// Ported to use with AVR
	
	SPDR = c;							// put byte in send-buffer
	while( !( SPSR & (1<<SPIF) ) )		
		;
	return SPDR;
}
// ---------------------------------------------------------------------------
/**
 * \brief Write to given register
 *
 * \param address Register address
 * \param data Value to write to given register
 */
void mcp2515_write_register(unsigned char address, unsigned char data) {
    
    //MCP2515_SS = 0;					// pull SS to low level
	RESET(MCP2515_CS);
   
    spi_transmit(MCP2515_CMD_WRITE);
    spi_transmit(address);
    spi_transmit(data);

    //MCP2515_SS = 1;					// release SS
	SET(MCP2515_CS);
}
// ---------------------------------------------------------------------------
/**
 * \brief Read from given register
 *
 * \param address Register address
 * \return register value
 */
unsigned char mcp2515_read_register(unsigned char address) {
    unsigned char data;

    //MCP2515_SS = 0;					// pull SS to low level
	RESET(MCP2515_CS);
   
    spi_transmit(MCP2515_CMD_READ);
    spi_transmit(address);   
    data = spi_transmit(0xff); 
  
    //MCP2515_SS = 1;					// release SS
	SET(MCP2515_CS);
   
    return data;
}
// ---------------------------------------------------------------------------
/**
 * \brief Modify bit of given register
 *
 * \param address Register address
 * \param mask Mask of bits to set
 * \param data Values to set
 *
 * This function works only on a few registers. Please check the datasheet!
 */
void mcp2515_bit_modify(unsigned char address, unsigned char mask, unsigned char data) {

    //MCP2515_SS = 0;					// pull SS to low level
	RESET(MCP2515_CS);
   
    spi_transmit(MCP2515_CMD_BIT_MODIFY);
    spi_transmit(address);
    spi_transmit(mask);
    spi_transmit(data);
   
    //MCP2515_SS = 1;					// release SS
	SET(MCP2515_CS);
}
// ---------------------------------------------------------------------------
/**
 * \ brief Initialize spi interface, 
 *   reset the MCP2515 and activate clock output signal
 */
bool mcp2515_init(void) {			// Origine - USB2CAN
    // init SPI
    //SSPSTAT = 0x40; 					// CKE=1
    //SSPCON1 = 0x22;
	// active SPI master interface - NEED To CHECK SPI SPEED with used CLOCK !!!!!
	// SPCR = (1<<SPE)|(1<<MSTR) | (0<<SPR1)|(1<<SPR0); // f_OSC/128 // SPSR = 0;
	SPCR = (1<<SPE)|(1<<MSTR); 				// SPI en & Master mode
	// switch to highest SPI frequency possible -- try Not to miss
    SPCR &= ~((1 << SPR1) | (1 << SPR0)); 	// Clock Frequency:f_OSC/4
    //SPSR |= (1 << SPI2X); 				// Doubled Clock Frequency:f_OSC/2 
	// reset MCP2515 by SW after this it is in configuration mode.

    //TRISBbits.TRISB4 = 1; // set TRIS of SDI
    //TRISCbits.TRISC6 = 0; // clear TRIS of SS
    //TRISCbits.TRISC7 = 0; // clear TRIS of SDO
    //TRISBbits.TRISB6 = 0; // clear TRIS of SCK
    //LATCbits.LATC6 = 1; // SS

	SET(MCP2515_CS);
	SET_OUTPUT(MCP2515_CS);
	
	RESET(P_SCK);
	RESET(P_MOSI);
	RESET(P_MISO);
	
	SET_OUTPUT(P_SCK);
	SET_OUTPUT(P_MOSI);
	SET_INPUT(P_MISO);
	
	SET_INPUT(MCP2515_INT);
	SET(MCP2515_INT);
	
    // reset device
    //LATCbits.LATC6 = 0; 		// SS - Chip select
	RESET(MCP2515_CS);
    spi_transmit(SPI_RESET);	// reset device
    SET(MCP2515_CS);
	//LATCbits.LATC6 = 1; 		// SS
    _delay_us(10);

    mcp2515_write_register(MCP2515_REG_CANCTRL, 0x85); // set config mode, clock prescaling 1:2 and clock output
    // configure filter
    mcp2515_write_register(MCP2515_REG_RXB0CTRL, 0x00); // use filter for standard and extended frames
    mcp2515_write_register(MCP2515_REG_RXB1CTRL, 0x00); // use filter for standard and extended frames
    // initialize filter mask
    mcp2515_write_register(MCP2515_REG_RXM0SIDH, 0x00);
    mcp2515_write_register(MCP2515_REG_RXM0SIDL, 0x00);
    mcp2515_write_register(MCP2515_REG_RXM0EID8, 0x00);
    mcp2515_write_register(MCP2515_REG_RXM0EID0, 0x00);
    mcp2515_write_register(MCP2515_REG_RXM1SIDH, 0x00);
    mcp2515_write_register(MCP2515_REG_RXM1SIDL, 0x00);
    mcp2515_write_register(MCP2515_REG_RXM1EID8, 0x00);
    mcp2515_write_register(MCP2515_REG_RXM1EID0, 0x00);
    mcp2515_write_register(MCP2515_REG_CANINTE, (1<<RX1IE)|(1<<RX0IE)); // RX interrupt
	
}
// -------------------------------------------------------------------------
bool mcp2515_init_XX(void){		// Origine - MiniOBD2 routine & modified

	SET(MCP2515_CS);
	SET_OUTPUT(MCP2515_CS);
	
	RESET(P_SCK);
	RESET(P_MOSI);
	RESET(P_MISO);
	
	SET_OUTPUT(P_SCK);
	SET_OUTPUT(P_MOSI);
	SET_INPUT(P_MISO);
	
	SET_INPUT(MCP2515_INT);
	SET(MCP2515_INT);
	
	// active SPI master interface
	// SPCR = (1<<SPE)|(1<<MSTR) | (0<<SPR1)|(1<<SPR0); // f_OSC/128 
	// SPSR = 0;
	SPCR = (1<<SPE)|(1<<MSTR); 				// SPI en & Master mode
	// switch to highest SPI frequency possible -- try Not to miss
    SPCR &= ~((1 << SPR1) | (1 << SPR0)); 	// Clock Frequency:f_OSC/4
    //SPSR |= (1 << SPI2X); 				// Doubled Clock Frequency:f_OSC/2 
	
	// reset MCP2515 by software reset.
	// After this he is in configuration mode.
	RESET(MCP2515_CS);
	spi_transmit(SPI_RESET);
	SET(MCP2515_CS);
	// wait a little bit until the MCP2515 has restarted
	_delay_us(10);
	// load CNF1..3 Register
	//RESET(MCP2515_CS);
	//spi_transmit(SPI_WRITE);
	//spi_transmit(CNF3);
	//spi_transmit((1<<PHSEG21));					// Bitrate 500 kbps at 16 MHz
	//spi_transmit((1<<BTLMODE)|(1<<PHSEG11));
	//spi_transmit(1<<BRP0); 
	
	// activate interrupts
	//spi_transmit((1<<RX1IE)|(1<<RX0IE));
	//SET(MCP2515_CS);
	//mcp2515_write_register(CNF1,(1<<BRP0));
	//mcp2515_write_register(CNF2,(1<<BTLMODE)|(1<<PHSEG11));
	//mcp2515_write_register(CNF3,(1<<PHSEG21));
	mcp2515_set_bittiming(MCP2515_TIMINGS_500K);
	mcp2515_write_register(MCP2515_REG_CANINTE, (1<<RX1IE)|(1<<RX0IE)); // RX interrupt
	
	// test if we could read back the value => is the chip accessible?
	if (mcp2515_read_register(CNF1) != (1<<BRP0)) {
		return false;
	}
	mcp2515_write_register(BFPCTRL, 0);		// deaktivate the RXnBF Pins (High Impedance State)
	mcp2515_write_register(TXRTSCTRL, 0);	// set TXnRTS as inputs
	mcp2515_write_register(RXB0CTRL, (1<<RXM1)|(1<<RXM0));	// turn off filters => receive any message
	mcp2515_write_register(RXB1CTRL, (1<<RXM1)|(1<<RXM0));
	mcp2515_write_register(CANCTRL, 0);		// reset device to normal mode
	return true;
}
// ---------------------------------------------------------------------------
/**
 * \brief Set filter mask of given SJA1000 register values
 *
 * \param amr0 Acceptence mask register 0
 * \param amr1 Acceptence mask register 1
 * \param amr2 Acceptence mask register 2
 * \param amr3 Acceptence mask register 3
 *
 * This function has only affect if mcp2515 is in configuration mode.
 * The filter mask is only set for the first 11 bit because of compatibility
 * issues between SJA1000 and MCP2515.
 */
void mcp2515_set_SJA1000_filter_mask(unsigned char amr0, unsigned char amr1, unsigned char amr2, unsigned char amr3) {

    // SJA1000 mask bit definition: 1 = accept without matching, 0 = do matching with acceptance code
    // MCP2515 mask bit definition: 0 = accept without matching, 1 = do matching with acceptance filter
    // -> invert mask

    // mask for filter 1
    mcp2515_write_register(MCP2515_REG_RXM0SIDH, ~amr0);
    mcp2515_write_register(MCP2515_REG_RXM0SIDL, (~amr1) & 0xE0);
    mcp2515_write_register(MCP2515_REG_RXM0EID8, 0x00);
    mcp2515_write_register(MCP2515_REG_RXM0EID0, 0x00);

    // mask for filter 2
    mcp2515_write_register(MCP2515_REG_RXM1SIDH, ~amr2);
    mcp2515_write_register(MCP2515_REG_RXM1SIDL, (~amr3) & 0xE0);
    mcp2515_write_register(MCP2515_REG_RXM1EID8, 0x00);
    mcp2515_write_register(MCP2515_REG_RXM1EID0, 0x00);

}
// ---------------------------------------------------------------------------
/**
 * \brief Set filter code of given SJA1000 register values
 *
 * \param amr0 Acceptence code register 0
 * \param amr1 Acceptence code register 1
 * \param amr2 Acceptence code register 2
 * \param amr3 Acceptence code register 3
 *
 * This function has only affect if mcp2515 is in configuration mode.
 */
void mcp2515_set_SJA1000_filter_code(unsigned char acr0, unsigned char acr1, unsigned char acr2, unsigned char acr3) {

    // acceptance code for filter 1
    mcp2515_write_register(MCP2515_REG_RXF0SIDH, acr0);
    mcp2515_write_register(MCP2515_REG_RXF0SIDL, (acr1) & 0xE0); // standard
    mcp2515_write_register(MCP2515_REG_RXF1SIDH, acr0);
    mcp2515_write_register(MCP2515_REG_RXF1SIDL, ((acr1) & 0xE0) | 0x80); // extended

    // acceptance code for filter 2
    mcp2515_write_register(MCP2515_REG_RXF2SIDH, acr2);
    mcp2515_write_register(MCP2515_REG_RXF2SIDL, (acr3) & 0xE0); // standard
    mcp2515_write_register(MCP2515_REG_RXF3SIDH, acr2);
    mcp2515_write_register(MCP2515_REG_RXF3SIDL, ((acr3) & 0xE0) | 0x80); // extended

    // fill remaining filters with zero
    mcp2515_write_register(MCP2515_REG_RXF4SIDH, 0x00);
    mcp2515_write_register(MCP2515_REG_RXF4SIDL, 0x00);
    mcp2515_write_register(MCP2515_REG_RXF5SIDH, 0x00);
    mcp2515_write_register(MCP2515_REG_RXF5SIDL, 0x00);
}
// ---------------------------------------------------------------------------
/**
 * \brief Set bit timing registers
 *
 * \param cnf1 Configuration register 1
 * \param cnf2 Configuration register 2
 * \param cnf3 Configuration register 3
 *
 * This function has only affect if mcp2515 is in configuration mode
 */
void mcp2515_set_bittiming(unsigned char cnf1, unsigned char cnf2, unsigned char cnf3) {

    mcp2515_write_register(MCP2515_REG_CNF1, cnf1);
    mcp2515_write_register(MCP2515_REG_CNF2, cnf2);
    mcp2515_write_register(MCP2515_REG_CNF3, cnf3);

}
// ---------------------------------------------------------------------------
/**
 * \brief Read status byte of MCP2515
 *
 * \return status byte of MCP2515
 */
unsigned char mcp2515_read_status(void){

    //MCP2515_SS = 0;					// pull SS to low level
	RESET(MCP2515_CS);
   
    spi_transmit(MCP2515_CMD_READ_STATUS);
    unsigned char status = spi_transmit(0xff);
   
    //MCP2515_SS = 1;					// release SS
	SET(MCP2515_CS);

    return status;
}
// ---------------------------------------------------------------------------
/**
 * \brief Read RX status byte of MCP2515
 *
 * \return RX status byte of MCP2515
 */
unsigned char mcp2515_rx_status(void) {

    //MCP2515_SS = 0;					// pull SS to low level
	RESET(MCP2515_CS);
   
    spi_transmit(MCP2515_CMD_RX_STATUS);
    unsigned char status = spi_transmit(0xff);
   
    //MCP2515_SS = 1;					// release SS
	SET(MCP2515_CS);

    return status;
}
// ---------------------------------------------------------------------------
/**
 * \brief Send given CAN message
 *
 * \ p_canmsg Pointer to can message to send
 * \ return 1 if transmitted successfully to MCP2515 transmit buffer, 
 * 			0 on error (= no free buffer available)
 */
unsigned char mcp2515_send_message(canmsg_t * p_canmsg) {

    unsigned char status = mcp2515_read_status();
    unsigned char address;

    // check length
    if (p_canmsg->length > 8) return 0;

    // get offest address of next free tx buffer
    if ((status & 0x04) == 0) {
        address = 0x00;
    } else if ((status & 0x10) == 0) {
        address = 0x02;
    } else if ((status & 0x40) == 0) {
        address = 0x04;
    } else {
        // no free transmit buffer
        return 0;
    }
    //MCP2515_SS = 0;					// pull SS to low level
	RESET(MCP2515_CS);
   
    spi_transmit(MCP2515_CMD_LOAD_TX | address);

    if (p_canmsg->flags.extended) {
        spi_transmit(p_canmsg->id >> 21);
        spi_transmit(((p_canmsg->id >> 13) & 0xe0) | ((p_canmsg->id >> 16) & 0x03) | 0x08);
        spi_transmit(p_canmsg->id >> 8);
        spi_transmit(p_canmsg->id);
    } else {
        spi_transmit(p_canmsg->id >> 3);
        spi_transmit(p_canmsg->id << 5);
        spi_transmit(0);
        spi_transmit(0);
    }
    // length and data
    if (p_canmsg->flags.rtr) {
        spi_transmit(p_canmsg->length | 0x40);
    } else {
        spi_transmit(p_canmsg->length);
        unsigned char i;
        for (i = 0; i < p_canmsg->length; i++) {
            spi_transmit(p_canmsg->data[i]);
        }
    }
	//MCP2515_SS = 1;					// release SS
	SET(MCP2515_CS);
    _delay_us(10);
    //MCP2515_SS = 0;					// pull SS to low level
	RESET(MCP2515_CS);

    if (address == 0) address = 1;
    spi_transmit(MCP2515_CMD_RTS | address);
    //MCP2515_SS = 1;					// release SS
	SET(MCP2515_CS);

    return 1;
}
// ---------------------------------------------------------------------------
/*
 * \brief Read out one can message from MCP2515
 *
 * \param p_canmsg Pointer to can message structure to fill
 * \return 1 on success, 0 if there is no message to read
 */
unsigned char mcp2515_receive_message(canmsg_t * p_canmsg) {

    unsigned char status = mcp2515_rx_status();
    unsigned char address;

    if (status & 0x40) {
        address = 0x00;
    } else if (status & 0x80) {
        address = 0x04;
    } else {
        // no message in receive buffer
        return 0;
    }
    p_canmsg->flags.rtr = (status >> 3) & 0x01;
    p_canmsg->flags.extended = (status >> 4) & 0x01;
    //MCP2515_SS = 0;					// pull SS to low level
	RESET(MCP2515_CS);
    spi_transmit(MCP2515_CMD_READ_RX | address);
    if (p_canmsg->flags.extended) {
	p_canmsg->id =  (unsigned long) spi_transmit(0xff) << 21;
        unsigned long temp = spi_transmit(0xff);
        p_canmsg->id |= (temp & 0xe0) << 13;
        p_canmsg->id |= (temp & 0x03) << 16;
        p_canmsg->id |= (unsigned long) spi_transmit(0xff) << 8;
        p_canmsg->id |= (unsigned long) spi_transmit(0xff);
    } else {
        p_canmsg->id =  (unsigned long) spi_transmit(0xff) << 3;
        p_canmsg->id |= (unsigned long) spi_transmit(0xff) >> 5;
        spi_transmit(0xff);
        spi_transmit(0xff);
    }
    // get length and data
    p_canmsg->length = spi_transmit(0xff) & 0x0f;
    if (!p_canmsg->flags.rtr) {
        unsigned char i;
        for (i = 0; i < p_canmsg->length; i++) {
            p_canmsg->data[i] = spi_transmit(0xff);
        }
    }
    //MCP2515_SS = 1;					// release SS
	SET(MCP2515_CS);
    if (address == 0) address = 1;
    else address = 2;
    mcp2515_bit_modify(MCP2515_REG_CANINTF, address, 0);

    return 1;
}
// ============================================================================
// 2013.10.14 : Following routines are added just for validation test
// ----------------------------------------------------------------------------
// check if there are any new messages waiting
unsigned char mcp2515_check_message(void){
	return (!IS_SET(MCP2515_INT));
}
// ------------------- Added by Turbo as old lib NOT support CAN filter -------
void mcp2515_static_filter(const prog_uint8_t *filter){
	// change to configuration mode
	mcp2515_bit_modify(MCP2515_REG_CANCTRL, 0xE0, (1<<REQOP2));	// REQOP2 = 7
	while ((mcp2515_read_register(MCP2515_REG_CANSTAT) & 0xE0) != (1<<7));
	mcp2515_write_register(MCP2515_REG_RXB0CTRL, (1<<BUKT));	// BUKT = 2
	mcp2515_write_register(MCP2515_REG_RXB1CTRL, 0);
    
	uint8_t i, j;
	for (i = 0; i < 0x30; i += 0x10){
		RESET(MCP2515_CS);
		spi_transmit(MCP2515_CMD_WRITE);
		spi_transmit(i);
		for (j = 0; j < 12; j++){
			if (i == 0x20 && j >= 0x08)		break;
			spi_transmit(pgm_read_byte(filter++));
		}
		SET(MCP2515_CS);
	}
	mcp2515_bit_modify(MCP2515_REG_CANCTRL, 0xE0, 0);
}
// ----------------------------------------------------------------------------
// check if there is a free buffer to send messages
bool mcp2515_check_free_buffer(void){
	unsigned char status = mcp2515_read_status();
	if ((status & 0x54) == 0x54) {
		// all buffers used
		return false;
	}
	return true;
}
// ---------------------------------------------------------------------------
