/*****************************************************************************
 File: usb2canAVR.c
 Description:
 This file contains the main logic of the USBtin project.
 USBtin is a simple USB to CAN interface. It uses the USB class CDC
 to connect to the host. Configuration and bus communication is
 handled over this CDC virtual comport.

 Authors and Copyright:
 (c) 2012, Thomas Fischl <tfischl@gmx.de>
 ****************************************************************************/
// ---------------------------------------------------------------------------
// 2013.10.15 : Porting to AVR & USB to RS232 (for BlueTooth or USB2Serial)
// ---------------------------------------------------------------------------
// V01 : 2013.10.25 - Initial working version
// V03 : 2013.11.02 - BUG fix mcp2515_init() : Wrong definition for BIT TIMING
// ---------------------------------------------------------------------------
#include <stdio.h>
#include <string.h>
#include <inttypes.h>
#include <stdint.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
//#include <avr/eeprom.h>
#include <util/delay.h>
#include <avr/wdt.h>
#include "uart.h"
#include "clock.h"			
#include "mcp2515.h"
#include "GKS0108Xr.h"					// with "Logo iTech"
// TO BE SET for ATMEGA ------------------------------------------------------
//#define hardware_setLED(value) LATBbits.LATB5 = value	// TBD
//#define hardware_getBLSwitch() !PORTAbits.RA3			// no use
//#define hardware_getMCP2515Int() !PORTCbits.RC2		// done in mcp2515.h
// ----------------------------------------------------------------------------
#define LINE_MAXLEN 			128
#define BELL 					7
#define CR 						13
#define LR 						10
// ----------------------------------------------------------------------------
#define STATE_CONFIG 			0
#define STATE_OPEN 				1
#define STATE_LISTEN 			2
// ----------------------------------------------------------------------------
#define VERSION_HARDWARE 		2
#define VERSION_FIRMWARE_MAJOR 	2
#define VERSION_FIRMWARE_MINOR 	1
// ----------------------------------------------------------------------------
//extern unsigned char COUNT_TCNT0;	
//extern unsigned short clock_msticker;
volatile unsigned char state = STATE_CONFIG;
unsigned char timestamping = 0, linepos = 0;
char line[LINE_MAXLEN];
extern int ReadKey, KeyDelay, menu;
// ----------------------------------------------------------------------------
void InitPort(void);
void DisplayStr(uint8_t x,uint8_t y, char * str,uint8_t size,uint8_t color);
void DisplayHex(int x, int y, char str[], int len, uint8_t fsize,uint8_t color);
void InitLogo(void);
unsigned char parseHex(char * line, unsigned char len, unsigned long * value);
char *sendHex(unsigned long value, unsigned char len);
void sendByteHex(unsigned char value);
unsigned char transmitStd(char *line);
void parseLine(char * line);
// ----------------------------------------------------------------------------
//  PORTC/D - GLCD, PORTG0/1/2 = SW
// ----------------------------------------------------------------------------
void InitPort(void){

	// PB0 (SS) should be high to enable SPI master
	//DDRB  = 0b11110111;				// PORTB - FatFs SPI & EN(CAN/CON & PIN)
	//PORTB = 0b11110101;
	//DDRC  = 0xFF;					// GLCD KS0108_CTRL_PORT
	//PORTC = 0x00;
	//DDRD  = 0xFF;					// GLCD KS0108_DATA_PORT
	//PORTD = 0x00;

	DDRF  = 0xFF;					// PORTF output
	PORTF = 0x00;
	DDRG  = 0xF0;					// PORTG 0.1.2.3 = SW In
	PORTG = 0x0F;					// in case of input, pull up - Bug fix

}
// ----------------------------------------------------------------------------
// To make easy to switch between LCDputsXY() and GLCD_WriteStr()
void DisplayStr(uint8_t x,uint8_t y, char * str,uint8_t size,uint8_t color){
	// GLCD_GoTo(0,1);						// x(col) = dot, y(row) = dot
	GLCD_WriteStr(x, y, str, size, color);	// SMALL/BIG, WHITE/BLACK
}
// ----------------------------------------------------------------------------
// To make easy to switch between LCDputsXY() and GLCD_WriteStr()
void DisplayHex(int x, int y, char str[], int len, uint8_t fsize,uint8_t color){
	// GLCD_GoTo(0,1);						// x(col) = dot, y(row) = dot
	unsigned char i;
	char s[21], buf[3];
	strcpy(s, "");
	if(len > 20)	len = 10;				// restrict the array size
	for(i=0; i<len; i++){
		sprintf(buf, "%02X", str[i]);
		strncat(s, buf, 2);
	}
	GLCD_WriteStr(x, y, s, fsize, color);	// SMALL/BIG, WHITE/BLACK
}
// ----------------------------------------------------------------------------
void InitLogo(void){

	GLCD_Bitmap(logo_itech, 0, 0, 128, 64);		_delay_ms(300);	
	GLCD_ClearScreen();	
	GLCD_Bitmap(logo_itech, 0, 0, 128, 64);		_delay_ms(100);
	//GLCD_Bitmap(logo_Jammer,0, 0, 128, 64);
	GLCD_ClearScreen();	
}
// ---------------------------------------------------------------------------
/**
 * Parse hex value of given string
 *
 * @param line Input string
 * @param len Count of characters to interpret
 * @param value Pointer to variable for the resulting decoded value
 * @return 0 on error, 1 on success
 */
unsigned char parseHex(char * line, unsigned char len, unsigned long * value) {
    *value = 0;
    while (len--) {
        if (*line == 0) return 0;
        *value <<= 4;
        if ((*line >= '0') && (*line <= '9')) {
           *value += *line - '0';
        } else if ((*line >= 'A') && (*line <= 'F')) {
           *value += *line - 'A' + 10;
        } else if ((*line >= 'a') && (*line <= 'f')) {
           *value += *line - 'a' + 10;
        } else return 0;
        line++;
    }
    return 1;
}
// ---------------------------------------------------------------------------
/**
 * Send given value as hexadecimal string
 *
 * @param value Value to send as hex over the UART
 * @param len Count of characters to produce
 */
char *sendHex(unsigned long value, unsigned char len) {

    static char s[21];
    s[len] = 0;									// fill NULL at the end of array

    while (len--) {
        unsigned char hex = value & 0x0f;
        if (hex > 9) hex = hex - 10 + 'A';
        else hex = hex + '0';
        s[len] = hex;
        value = value >> 4;
    }
    //usb_putstr(s);
	uart_puts(s);
	return s;
}
// ---------------------------------------------------------------------------
/**
 * Send given byte value as hexadecimal string
 *
 * @param value Byte value to send over UART
 */
void sendByteHex(unsigned char value) {

    sendHex(value, 2);
}
// ---------------------------------------------------------------------------
/**
 * Interprets given line and transmit can message
 *
 * @param line Line string which contains the transmit command
 */
unsigned char transmitStd(char *line) {
    canmsg_t canmsg;
    unsigned long temp;
    unsigned char idlen;

    canmsg.flags.rtr = ((line[0] == 'r') || (line[0] == 'R'));
	//DisplayStr(8, 40, "Sending CAN message", SMALL, WHITE);
    // upper case -> extended identifier
    if (line[0] < 'Z') {
        canmsg.flags.extended = 1;
        idlen = 8;
    } else {
        canmsg.flags.extended = 0;
        idlen = 3;
    }
    if (!parseHex(&line[1], idlen, &temp)) return 0;
    canmsg.id = temp;
    if (!parseHex(&line[1 + idlen], 1, &temp)) return 0;
    canmsg.length = temp;
    if (!canmsg.flags.rtr) {
        unsigned char i;
        for (i = 0; i < canmsg.length; i++) {
            if (!parseHex(&line[idlen + 2 + i*2], 2, &temp)) return 0;
            canmsg.data[i] = temp;
        }
    }
	//DisplayStr(8, 48, "CAN message sent", SMALL, WHITE);
    return mcp2515_send_message(&canmsg);
}
// ---------------------------------------------------------------------------
/**
 * Parse given command line
 *
 * @param line Line string to parse
 */
void parseLine(char * line) {

	unsigned long cnf1, cnf2, cnf3;
	unsigned long am0, am1, am2, am3;
	unsigned long ac0, ac1, ac2, ac3;
	unsigned long address, data, stamping;
	unsigned char flags, status, value, result = BELL;

    switch (line[0]) {
        case 'S': // Setup with standard CAN bitrates
            if (state == STATE_CONFIG){
                switch (line[1]) {
					// follwoing timings are NOT checked & could be wrong
                    case '0': mcp2515_set_bittiming(MCP2515_TIMINGS_10K);  result = CR; break;
                    case '1': mcp2515_set_bittiming(MCP2515_TIMINGS_20K);  result = CR; break;
                    case '2': mcp2515_set_bittiming(MCP2515_TIMINGS_50K);  result = CR; break;
                    case '3': mcp2515_set_bittiming(MCP2515_TIMINGS_100K); result = CR; break;
                    case '7': mcp2515_set_bittiming(MCP2515_TIMINGS_800K); result = CR; break;
					// Flowing timings are modified & 500K is checked only
					case '4': mcp2515_set_bittiming(MCP2515_TIMINGS_125K); result = CR; break;
                    case '5': mcp2515_set_bittiming(MCP2515_TIMINGS_250K); result = CR; break;
                    case '6': mcp2515_set_bittiming(MCP2515_TIMINGS_500K); result = CR; break;
                    case '8': mcp2515_set_bittiming(MCP2515_TIMINGS_1M);   result = CR; break;
                }
            }
            break;
        case 's': // Setup with user defined timing settings for CNF1/CNF2/CNF3
            if (state == STATE_CONFIG){
                if (parseHex(&line[1], 2, &cnf1) && parseHex(&line[3], 2, &cnf2) && parseHex(&line[5], 2, &cnf3)) {
                    mcp2515_set_bittiming(cnf1, cnf2, cnf3);
                    result = CR;
                }
            }
            break;
        case 'G': // Read given MCP2515 register
            if (parseHex(&line[1], 2, &address)) {
                value = mcp2515_read_register(address);
				sendByteHex(value);
                result = CR;
            }
            break;
        case 'W': // Write given MCP2515 register
            if (parseHex(&line[1], 2, &address) && parseHex(&line[3], 2, &data)) {
                mcp2515_write_register(address, data);
                result = CR;
            }
            break;
        case 'V': // Get versions
            //usb_putch('V');
			uart_putc('V');
            sendByteHex(VERSION_HARDWARE);
            sendByteHex(VERSION_FIRMWARE_MAJOR);
            result = CR;
            break;
        case 'v': // Get firmware version
            //usb_putch('v');
			uart_putc('v');
            sendByteHex(VERSION_FIRMWARE_MAJOR);
            sendByteHex(VERSION_FIRMWARE_MINOR);
            result = CR;
            break;
        case 'N': // Get serial number
            //usb_putch('N');
			uart_putc('N');
            sendHex(0xFFFF, 4);
            result = CR;
            break;
        case 'O': // Open CAN channel		// set normal operating mode
            if (state == STATE_CONFIG){
				mcp2515_bit_modify(MCP2515_REG_CANCTRL, 0xE0, 0x00);
                clock_start();			
                state = STATE_OPEN;
                result = CR;
            }
            break;
        case 'l': // Loop-back mode			// set loop-back
            if (state == STATE_CONFIG){
				mcp2515_bit_modify(MCP2515_REG_CANCTRL, 0xE0, 0x40);
                state = STATE_OPEN;
                result = CR;
            }
            break;
        case 'L': // Open & set CAN channel in listen-only mode	
            if (state == STATE_CONFIG){
				mcp2515_bit_modify(MCP2515_REG_CANCTRL, 0xE0, 0x60);
                state = STATE_LISTEN;
                result = CR;
            }
            break;
        case 'C': // Close CAN channel		// set configuration mode
            if (state != STATE_CONFIG){
				mcp2515_bit_modify(MCP2515_REG_CANCTRL, 0xE0, 0x80);
                state = STATE_CONFIG;
                result = CR;
            }
            break;
        case 'r': // Receive standard RTR (11 bit) frame
        case 'R': // Receive extended RTR (29 bit) frame
        case 't': // Transmit standard (11 bit) frame
        case 'T': // Transmit extended (29 bit) frame
            if (state == STATE_OPEN){
                if (transmitStd(line)) {
                    if (line[0] < 'Z') 	uart_putc('Z');	//usb_putch('Z');
                    else 				uart_putc('z');	//usb_putch('z');
                    result = CR;
                }
            }
            break;
        case 'F': // Read status flags
            flags = mcp2515_read_register(MCP2515_REG_EFLG);
            status = 0;
            if (flags & 0x01) status |= 0x04; // error warning
            if (flags & 0xC0) status |= 0x08; // data overrun
            if (flags & 0x18) status |= 0x20; // passive error
            if (flags & 0x20) status |= 0x80; // bus error
            //usb_putch('F');
			uart_putc('F');
            sendByteHex(status);
            result = CR;
            break;
        case 'Z': // Set time stamping
            if (parseHex(&line[1], 1, &stamping)) {
				timestamping = (stamping != 0);
                result = CR;
            }
            break;
        case 'm': // Set accpetance filter mask
            if (state == STATE_CONFIG){
                if (parseHex(&line[1], 2, &am0) && parseHex(&line[3], 2, &am1) && parseHex(&line[5], 2, &am2) && parseHex(&line[7], 2, &am3)) {
                    mcp2515_set_SJA1000_filter_mask(am0, am1, am2, am3);
                    result = CR;
                }
            }
            break;
        case 'M': // Set accpetance filter code
            if (state == STATE_CONFIG){
                if (parseHex(&line[1], 2, &ac0) && parseHex(&line[3], 2, &ac1) && parseHex(&line[5], 2, &ac2) && parseHex(&line[7], 2, &ac3)) {
                    mcp2515_set_SJA1000_filter_code(ac0, ac1, ac2, ac3);
                    result = CR;
                }
            }
            break;
        default:	break;
    }
	//usb_putch(result);
	uart_putc(result);
}
// ---------------------------------------------------------------------------
/**
 * Main function. Entry point for USBtin application.
 * Handles initialization and the the main processing loop.
 */
int main(void){
	unsigned short timestamp;
	unsigned char idlen, i, ch;
	char type;
	canmsg_t canmsg;
    // switch (back) to external clock (if fail-safe-monitor switched to internal)
    // OSCCON = 0x30; // <---------- for PIC not for ATMEGA
	// disable all analog pin functions, set led pin to output
    //ANSEL = 0;
    //ANSELH = 0;
    //TRISBbits.TRISB5 = 0;
	//hardware_setLED(0);

	// @ 16MHz : 115.2K=-3.5%(with=8), 57.6K=2%, 38.4K/19.2K=0.2%
	// Using 14.7456MHz - all 0%
	// Setting U2Xn = 1 at UCSRnA - it will doubled transmit speed
	// @ 16MHz with U2Xn=1: 115.2K/57.6K=2.1%, 38.4K=0.2%, 19.2K=-0.8%
	uart_init(UART_BAUD_SELECT(57600UL, 16000000));	// Choose low error speed
	InitPort();
	clock_start();
	sei();											// Enable ISR !!!!!!
	GLCD_Initalize();		
	GLCD_ClearScreen();
    // initialize MCP2515 (reset and clock setup)
	mcp2515_init();
	//InitLogo();
	DisplayStr(0, 0, "USB2CAN AVR", SMALL, WHITE);
	uart_puts("USB2CAN AVR");
    // initialize modules
    //clock_init(); 	
    //usb_init();
	DisplayStr(60, 0, "In loop ...", SMALL, WHITE);
    while(1){
        // do module processing
        //usb_process();		// No need as UART INT will do RECV job
        //clock_process();		// no need as it is done in timer InitISR()
        //if (usb_chReceived()) {
		// Receive ch from UART & collect the data until EOL is indicated	
		if (uart_chReceived()) {
			DisplayStr(8, 24, "UART msg ...", SMALL, WHITE);
			//ch = (unsigned char)(uart_getc());	//usb_getch();
			ch = uart_getc();	//usb_getch();
			if (ch == CR) {
				line[linepos] = 0;
				parseLine(line);
				DisplayStr(0, 8, "                    ", SMALL, WHITE);
				DisplayStr(0, 8, line, SMALL, WHITE);	// disp command
				linepos = 0;
			} else if (ch != LR) {
				line[linepos] = ch;
				if (linepos < LINE_MAXLEN - 1) linepos++;
			}
		}
		// Handles INT requests of MCP2515 : receive & print the message
		//if ((state != STATE_CONFIG) && (hardware_getMCP2515Int())) {
		//if (mcp2515_check_message()){
		if ((state != STATE_CONFIG) && (!IS_SET(MCP2515_INT))) {
			DisplayStr(8, 16, "RECV:CAN msg ...", SMALL, WHITE);
			mcp2515_receive_message(&canmsg);
			timestamp = clock_getMS();	
			if (canmsg.flags.rtr) type = 'r';
			else type = 't';
			if (canmsg.flags.extended) {
				type -= 'a' - 'A';
				idlen = 8;
			} else {
				idlen = 3;
			}
			//usb_putch(type);
			uart_putc(type);
			sendHex(canmsg.id, idlen);
			sendHex(canmsg.length, 1);
			if (!canmsg.flags.rtr) {
				for (i = 0; i < canmsg.length; i++) {
					sendHex(canmsg.data[i], 2);
				}
			}
			if (timestamping) {			// Global variable
				sendHex(timestamp, 4);
			}
			//usb_putch(CR);
			uart_putc(CR);
			DisplayStr(8, 24, "                    ", SMALL, WHITE);
			//DisplayStr(8, 24, canmsg.data, SMALL, WHITE);
			DisplayHex(8, 24, canmsg.data, 8, SMALL, WHITE);
		}
		// led signaling
		//hardware_setLED(state != STATE_CONFIG);
		// jump into bootloader, if jumper is closed
		//if (hardware_getBLSwitch()) {
		//    UCON = 0;
		//    _delay(1000);
		//    RESET();
		//}	
    }
}
// ---------------------------------------------------------------------------
/*****************************************************************************
 Device: PIC18F14K50
 Compiler: HI-TECH C PRO for the PIC18 MCU Family (Lite)  V9.65

 License:
 This file is open source. You can use it or parts of it in own
 open source projects. For closed or commercial projects you have to
 contact the authors listed above to get the permission for using
 this source code.

 Change History:
  Rev   Date        Description
  1.0   2011-12-18  Initial release
  1.1   2012-03-09  Minor fixes, code cleanup
  1.2   2013-01-11  Added filter function (command 'm' and 'M')
                    Added write register function (command 'W')
 ****************************************************************************/



